#!/usr/bin/perl

package Cognition::Export::Feed;
use Cognition::Export;
use Cognition::Export::BlogEntry;
use HTML::Entities qw(encode_entities_numeric);
use MIME::Base64;
use URI;

sub is
{
	my $subject = shift;
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://purl.org/rss/1.0/channel')
			||  ($type eq 'urn:ietf:rfc:4287#feed'));			
	}
	
	return 1 if (firstOf($subject,
		'http://purl.org/rss/1.0/item',
		'http://purl.org/rss/1.0/items',
		'urn:ietf:rfc:4287#entry'));
	
	return;
}

sub outputs
{
	return [
		{
			code  => 'Atom',
			name  => 'Atom Feed',
			type  => 'application/atom+xml'
		}
	];
}

sub outAtom
{
	my $triples = shift;
	my $subject = shift;
	my $S = $triples->{$subject};
	my $rv = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
	$rv .= "<feed xmlns=\"http://www.w3.org/2005/Atom\">\n";
	$rv .= "\t<generator>cognition</generator>\n";
	
	my $link = firstOf($S,
		'urn:ietf:rfc:4287#link',
		'http://purl.org/rss/1.0/link',
		'http://purl.org/dc/terms/identifier',
		'http://purl.org/dc/elements/1.1/identifier');
	$rv .= "\t<id>".encode_entities_numeric($link)."</id>\n";
	$rv .= "\t<link rel=\"alternate\" href=\"".encode_entities_numeric($link)."\" />\n";

	my $title = firstOf($S,
		'urn:ietf:rfc:4287#title',
		'http://purl.org/rss/1.0/title',
		'http://purl.org/dc/terms/title',
		'http://purl.org/dc/elements/1.1/title');
	$title = 'Untitled' if (!$title);
	$rv .= "\t<title>".encode_entities_numeric($title)."</title>\n";
	
	my $subtitle = firstOf($S,
		'urn:ietf:rfc:4287#subtitle',
		'http://purl.org/dc/terms/abstract');
	$rv .= "\t<subtitle>".encode_entities_numeric($subtitle)."</subtitle>\n"
		if ($subtitle);

	my @authors = allOfObj($S,
		'urn:ietf:rfc:4287#author',
		'http://purl.org/dc/terms/creator',
		'http://purl.org/dc/elements/1.1/creator',
		'http://xmlns.com/foaf/0.1/maker');
	foreach my $author (@authors)
	{
		if ($author->type && Cognition::Export::Contact::is($triples->{$author->toString}))
		{
			$rv .= Cognition::Export::Contact::_AtomPerson(
				$triples,
				$author->toString,
				"\t<author>");
		}
		elsif ($author->type)
		{
			$rv .= "\t<author><url>".$author->toString."</url></author>\n";
		}
		else
		{
			$rv .= "\t<author><name>".$author->toString."</name></author>\n";
		}
	}

	my @contributors = allOfObj($S,
		'urn:ietf:rfc:4287#contributor',
		'http://purl.org/dc/terms/contributor',
		'http://purl.org/dc/elements/1.1/contributor');
	foreach my $contributor (@contributors)
	{
		if ($contributor->type && Cognition::Export::Contact::is($triples->{$contributor->toString}))
		{
			$rv .= Cognition::Export::Contact::_AtomPerson(
				$triples,
				$contributor->toString,
				"\t<contributor>");
		}
		elsif ($contributor->type)
		{
			$rv .= "\t<contributor><url>".$contributor->toString."</url></contributor>\n";
		}
		else
		{
			$rv .= "\t<contributor><name>".$contributor->toString."</name></contributor>\n";
		}
	}

	$rv .= "\t<updated>UPDATED_PLACEHOLDER_99754106633f94d350db34d548d6091a</updated>\n";

	my @cats = allOfObj($S, 'urn:ietf:rfc:4287#category');
	foreach my $cat (@cats)
	{
		my ($tag, $scheme, $label);
		if ($cat->type && defined $triples->{$cat->toString})
		{
			$tag    = firstOf($triples->{$cat->toString},
				'http://microformats.org/wiki/rel-tag#tag',
				'http://purl.org/dc/terms/title',
				'http://purl.org/dc/elements/1.1/title');
			$scheme = firstOf($triples->{$cat->toString},
				'http://microformats.org/wiki/rel-tag#tagspace');
			$label  = firstOf($triples->{$cat->toString},
				'http://microformats.org/wiki/rel-tag#label');
		}
		$tag   = $cat->toString if (!$tag);
		$label = $cat->toString if (!$label);
		
		$rv .= "\t<category term=\"".encode_entities_numeric($tag)."\" ";
		$rv .= "scheme=\"".encode_entities_numeric($scheme)."\" "
			if (length $scheme);
		$rv .= "label=\"".encode_entities_numeric($label)."\" "
			if (length $label);
		$rv .= "/>\n";
	}

	my $rights = firstOf($S,
		'urn:ietf:rfc:4287#rights',
		'http://purl.org/dc/terms/rights',
		'http://purl.org/dc/elements/1.1/rights');
	$rv .= "\t\t<rights>".encode_entities_numeric($rights)."</rights>\n"
		if ($rights);
	
	my $updatedLatest;
	foreach my $e (allOf($S, 'http://purl.org/rss/1.0/item',
		'urn:ietf:rfc:4287#entry'))
	{
		$rv .= Cognition::Export::BlogEntry::_outAtom($triples, $e, $updatedLatest);
	}

	my $updated = firstOf($S,
		'urn:ietf:rfc:4287#updated',
		'http://purl.org/dc/terms/modified',
		'http://purl.org/dc/terms/date',
		'http://purl.org/dc/elements/1.1/date');
	if (!$updated)
	{
		$updated = $updatedLatest;
	}
	if ($updated)
	{
		$updated =~ s/(\d\d)(\d\d)$/$1:$2/;
		$rv =~ s/UPDATED_PLACEHOLDER_99754106633f94d350db34d548d6091a/$updated/;
	}
	
	$rv .= "</feed>\n";
	return $rv;
}

1;
