#!/usr/bin/perl

######################################################################
package Cognition::uF::RelLicense;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @rv;
	
	# Unqualified 'license' rel.
	my @nodes = Cognition::HTMLParser::Utils::searchRel('license', $within);
	
	# Various namespace-qualified alternatives...
	push @nodes, Cognition::HTMLParser::Utils::searchRel($page->_fq2pfx('http://www.w3.org/1999/xhtml/vocab#license'), $within);
	push @nodes, Cognition::HTMLParser::Utils::searchRel($page->_fq2pfx('http://web.resource.org/cc/license'), $within);
	push @nodes, Cognition::HTMLParser::Utils::searchRel($page->_fq2pfx('http://purl.org/dc/terms/license'), $within);
	
	foreach my $l (@nodes)
	{
		my $l_parsed = parse($page, $l);
		push @rv, $l_parsed;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if ($this->{license})
	{
		push @rv, $this->{license};
	}
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub metadata_ns
{
	my $this = shift;
	return { 'CC' => { nsurl=>'http://web.resource.org/cc/',title=>'Creative Commons' } };
} #/sub namespace


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'http://web.resource.org/cc/';

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	# Patterns for simple data.
	my @_SINGLES  = qw(title);
	my @_SINGLESR = qw();
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		$subject->addVal('http://purl.org/dc/terms/'.$property, $val);
	}

	# Resource-type properties which exist as singles.
	foreach my $property (@_SINGLESR)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property}, 1);
		$subject->addVal($ns.$property, $val);
	}		
	
	return $subject;
	
} #/sub rdf_subject


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $t_node = $rv->{'_dom'}->cloneNode(1);

	my $id = $t_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	$rv->{'license'} = $page->abs_url($t_node->getAttribute('href'));
	if (length $t_node->getAttribute('title'))
		{ $rv->{'title'} = $t_node->getAttribute('title'); }
	else
		{ $rv->{'title'} = Cognition::HTMLParser::Utils::STRINGIFY($t_node); }

	bless $rv;
	return $rv;	
} #/sub parse


1; #/package Cognition::uF::RelLicense




